@@ -1,7 +1,7 @@
 ;;; tempo.el --- Flexible template insertion
 
 ;; Copyright (C) 1994, 1995, 2001, 2002, 2003, 2004,
-;;   2005, 2006, 2007 Free Software Foundation, Inc.
+;;   2005, 2006, 2007, 2008, 2009 Free Software Foundation, Inc.
 
 ;; Author: David K}gedal <davidk@lysator.liu.se>
 ;; Created: 16 Feb 1994
@@ -10,10 +10,10 @@
 
 ;; This file is part of GNU Emacs.
 
-;; GNU Emacs is free software; you can redistribute it and/or modify
+;; GNU Emacs is free software: you can redistribute it and/or modify
 ;; it under the terms of the GNU General Public License as published by
-;; the Free Software Foundation; either version 2, or (at your option)
-;; any later version.
+;; the Free Software Foundation, either version 3 of the License, or
+;; (at your option) any later version.
 
 ;; GNU Emacs is distributed in the hope that it will be useful,
 ;; but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -21,9 +21,7 @@
 ;; GNU General Public License for more details.
 
 ;; You should have received a copy of the GNU General Public License
-;; along with GNU Emacs; see the file COPYING.  If not, write to the
-;; Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
-;; Boston, MA 02110-1301, USA.
+;; along with GNU Emacs.  If not, see <http://www.gnu.org/licenses/>.
 
 ;;; Commentary:
 
@@ -84,7 +82,7 @@
 
 ;; If the 'o is the first element in a template, strange things can
 ;; happen when the template is inserted at the beginning of a
-;; line. This is due to strange behaviour in open-line. But it should
+;; line. This is due to strange behavior in open-line. But it should
 ;; be easily avoided.
 
 ;; The 'o tag is also a problem when including the region. This will
@@ -116,14 +114,14 @@
   :group 'tools)
 
 (defcustom tempo-interactive nil
-  "*Prompt user for strings in templates.
+  "Prompt user for strings in templates.
 If this variable is non-nil, `tempo-insert' prompts the
 user for text to insert in the templates."
   :type 'boolean
   :group 'tempo)
 
 (defcustom tempo-insert-region nil
-  "*Automatically insert current region when there is a `r' in the template
+  "Automatically insert current region when there is a `r' in the template
 If this variable is nil, `r' elements will be treated just like `p'
 elements, unless the template function is given a prefix (or a non-nil
 argument).  If this variable is non-nil, the behavior is reversed.
@@ -133,13 +131,13 @@ In Transient Mark mode, this option is unused."
   :group 'tempo)
 
 (defcustom tempo-show-completion-buffer t
-  "*If non-nil, show a buffer with possible completions, when only
+  "If non-nil, show a buffer with possible completions, when only
 a partial completion can be found."
   :type 'boolean
   :group 'tempo)
 
 (defcustom tempo-leave-completion-buffer nil
-  "*If nil, a completion buffer generated by \\[tempo-complete-tag]
+  "If nil, a completion buffer generated by \\[tempo-complete-tag]
 disappears at the next keypress; otherwise, it remains forever."
   :type 'boolean
   :group 'tempo)
@@ -315,9 +313,9 @@ mode, ON-REGION is ignored and assumed true if the region is active."
 	    (goto-char tempo-region-start))
 	(save-excursion
 	  (tempo-insert-mark (point-marker))
-	  (mapcar (function (lambda (elt)
-			      (tempo-insert elt on-region)))
-		  (symbol-value template))
+	  (mapc (function (lambda (elt)
+			    (tempo-insert elt on-region)))
+		(symbol-value template))
 	  (tempo-insert-mark (point-marker)))
 	(tempo-forward-mark))
     (tempo-forget-insertions)
@@ -460,10 +458,10 @@ never prompted."
   "Tries all the user-defined element handlers in `tempo-user-elements'."
   ;; Sigh... I need (some list)
   (catch 'found
-    (mapcar (function (lambda (handler)
-			(let ((result (funcall handler element)))
-			  (if result (throw 'found result)))))
-	    tempo-user-elements)
+    (mapc (function (lambda (handler)
+		      (let ((result (funcall handler element)))
+			(if result (throw 'found result)))))
+	  tempo-user-elements)
     (throw 'found nil)))
 
 ;;;
@@ -556,7 +554,7 @@ and insert the results."
   "Jump to the next mark in `tempo-forward-mark-list'."
   (interactive)
   (let ((next-mark (catch 'found
-		     (mapcar
+		     (mapc
 		      (function
 		       (lambda (mark)
 			 (if (< (point) mark)
@@ -575,7 +573,7 @@ and insert the results."
   (interactive)
   (let ((prev-mark (catch 'found
 		     (let (last)
-		       (mapcar
+		       (mapc
 			(function
 			 (lambda (mark)
 			   (if (<= (point) mark)
@@ -765,5 +763,5 @@ space bar, and looks something like this:
 
 (provide 'tempo)
 
-;;; arch-tag: b3c0ee36-db3b-47bc-875f-091b4e27a063
+;; arch-tag: b3c0ee36-db3b-47bc-875f-091b4e27a063
 ;;; tempo.el ends here
