@@ -3,13 +3,13 @@
 #include "copyright.h"
 
 /*
-Copyright (C) 1993, 1994, 2001, 2002, 2003, 2004, 2005, 2006,
-  2007, 2008 Free Software Foundation, Inc.
+Copyright (C) 1993, 1994, 2001, 2002, 2003, 2004, 2005, 2006, 2007,
+  2008, 2009  Free Software Foundation, Inc.
 
-This program is free software; you can redistribute it and/or modify
+This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
-the Free Software Foundation; either version 3, or (at your option)
-any later version.
+the Free Software Foundation, either version 3 of the License, or
+(at your option) any later version.
 
 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -17,9 +17,7 @@ MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
-along with this program; see the file COPYING.  If not, write to the
-Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
-Boston, MA 02110-1301, USA. */
+along with this program.  If not, see <http://www.gnu.org/licenses/>.  */
 
 
 /*
@@ -52,7 +50,6 @@ Boston, MA 02110-1301, USA. */
 #include "../src/bitmaps/stipple.xbm"
 
 #else
-#ifndef VMS
 
 #include <X11/bitmaps/dimple1>
 #include <X11/bitmaps/dimple3>
@@ -68,23 +65,6 @@ Boston, MA 02110-1301, USA. */
 #include <X11/bitmaps/cntr_ptrmsk>
 #include <X11/bitmaps/stipple>
 
-#else
-
-#include "[-.src.bitmaps]dimple1.xbm"
-#include "[-.src.bitmaps]dimple3.xbm"
-#include "[-.src.bitmaps]gray1.xbm"
-#include "[-.src.bitmaps]gray3.xbm"
-#include "[-.src.bitmaps]crosswv.xbm"
-
-#include "[-.src.bitmaps]leftptr.xbm"
-#include "[-.src.bitmaps]leftpmsk.xbm"
-#include "[-.src.bitmaps]rtptr.xbm"
-#include "[-.src.bitmaps]rtpmsk.xbm"
-#include "[-.src.bitmaps]cntrptr.xbm"
-#include "[-.src.bitmaps]cntrpmsk.xbm"
-#include "[-.src.bitmaps]stipple.xbm"
-
-#endif /* VMS */
 #endif /* not EMACS_BITMAP_FILES */
 
 #define DEF_FREEZE		0
