@@ -1,13 +1,13 @@
 /* Definitions needed by most editing commands.
    Copyright (C) 1985, 1994, 2001, 2002, 2003, 2004,
-                 2005, 2006, 2007  Free Software Foundation, Inc.
+                 2005, 2006, 2007, 2008, 2009  Free Software Foundation, Inc.
 
 This file is part of GNU Emacs.
 
-GNU Emacs is free software; you can redistribute it and/or modify
+GNU Emacs is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
-the Free Software Foundation; either version 2, or (at your option)
-any later version.
+the Free Software Foundation, either version 3 of the License, or
+(at your option) any later version.
 
 GNU Emacs is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -15,9 +15,7 @@ MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
-along with GNU Emacs; see the file COPYING.  If not, write to
-the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
-Boston, MA 02110-1301, USA.  */
+along with GNU Emacs.  If not, see <http://www.gnu.org/licenses/>.  */
 
 
 #define Ctl(c) ((c)&037)
@@ -45,12 +43,12 @@ extern Lisp_Object Vminibuffer_local_must_match_map;
 
 /* keymap used for minibuffers when doing completion in filenames
    and require a match */
-extern Lisp_Object Vminibuffer_local_must_match_filename_map;
+extern Lisp_Object Vminibuffer_local_filename_must_match_map;
 
-/* Last character of last key sequence.  */
-extern Lisp_Object last_command_char;
+/* Last input event read as a command.  */
+extern Lisp_Object last_command_event;
 
-/* Last input character read as a command, not counting menus
+/* Last input event read as a command, not counting menus
    reached by the mouse.  */
 extern Lisp_Object last_nonmenu_event;
 
