@@ -1,12 +1,13 @@
 /* Calculate what line insertion or deletion to do, and do it,
-   Copyright (C) 1985, 1986, 1990, 1993, 1994 Free Software Foundation, Inc.
+   Copyright (C) 1985, 1986, 1990, 1993, 1994, 2001, 2002, 2003, 2004,
+                 2005, 2006, 2007, 2008, 2009  Free Software Foundation, Inc.
 
 This file is part of GNU Emacs.
 
-GNU Emacs is free software; you can redistribute it and/or modify
+GNU Emacs is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
-the Free Software Foundation; either version 2, or (at your option)
-any later version.
+the Free Software Foundation, either version 3 of the License, or
+(at your option) any later version.
 
 GNU Emacs is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -14,23 +15,19 @@ MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
-along with GNU Emacs; see the file COPYING.  If not, write to
-the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-Boston, MA 02111-1307, USA.  */
+along with GNU Emacs.  If not, see <http://www.gnu.org/licenses/>.  */
 
 
 #include <config.h>
 #include <stdio.h>
 #include <string.h>
-#include "termchar.h"
 #include "lisp.h"
+#include "termchar.h"
 #include "dispextern.h"
 #include "keyboard.h"
 #include "frame.h"
 #include "window.h"
-
-#define max(a, b) ((a) > (b) ? (a) : (b))
-#define min(a, b) ((a) < (b) ? (a) : (b))
+#include "termhooks.h"
 
 /* All costs measured in characters.
    So no cost can exceed the area of a frame, measured in characters.
@@ -60,10 +57,12 @@ struct matrix_elt
     unsigned char writecount;
   };
 
-static void do_direct_scrolling P_ ((struct glyph_matrix *,
+static void do_direct_scrolling P_ ((struct frame *,
+                                     struct glyph_matrix *,
 				     struct matrix_elt *,
 				     int, int));
-static void do_scrolling P_ ((struct glyph_matrix *,
+static void do_scrolling P_ ((struct frame *,
+                              struct glyph_matrix *,
 			      struct matrix_elt *,
 			      int, int));
 
@@ -92,34 +91,35 @@ calculate_scrolling (frame, matrix, window_size, lines_below,
      FRAME_PTR frame;
      /* matrix is of size window_size + 1 on each side.  */
      struct matrix_elt *matrix;
-     int window_size;
+     int window_size, lines_below;
      int *draw_cost;
      int *old_hash;
      int *new_hash;
      int free_at_end;
 {
   register int i, j;
-  int frame_height = FRAME_HEIGHT (frame);
+  int frame_lines = FRAME_LINES (frame);
   register struct matrix_elt *p, *p1;
   register int cost, cost1;
 
-  int lines_moved = window_size + (scroll_region_ok ? 0 : lines_below);
+  int lines_moved = window_size
+    + (FRAME_SCROLL_REGION_OK (frame) ? 0 : lines_below);
   /* first_insert_cost[I] is the cost of doing the first insert-line
      at the i'th line of the lines we are considering,
      where I is origin 1 (as it is below).  */
   int *first_insert_cost
-    = &FRAME_INSERT_COST (frame)[frame_height - 1 - lines_moved];
+    = &FRAME_INSERT_COST (frame)[frame_lines - 1 - lines_moved];
   int *first_delete_cost
-    = &FRAME_DELETE_COST (frame)[frame_height - 1 - lines_moved];
+    = &FRAME_DELETE_COST (frame)[frame_lines - 1 - lines_moved];
   int *next_insert_cost
-    = &FRAME_INSERTN_COST (frame)[frame_height - 1 - lines_moved];
+    = &FRAME_INSERTN_COST (frame)[frame_lines - 1 - lines_moved];
   int *next_delete_cost
-    = &FRAME_DELETEN_COST (frame)[frame_height - 1 - lines_moved];
+    = &FRAME_DELETEN_COST (frame)[frame_lines - 1 - lines_moved];
 
   /* Discourage long scrolls on fast lines.
      Don't scroll nearly a full frame height unless it saves
      at least 1/4 second.  */
-  int extra_cost = baud_rate / (10 * 4 * FRAME_HEIGHT (frame));
+  int extra_cost = baud_rate / (10 * 4 * FRAME_LINES (frame));
 
   if (baud_rate <= 0)
     extra_cost = 1;
@@ -235,15 +235,16 @@ calculate_scrolling (frame, matrix, window_size, lines_below,
 /* Perform insert-lines and delete-lines operations on CURRENT_MATRIX
    according to the costs in MATRIX, using the general scrolling
    method that is used if the terminal does not support the setting of
-   scroll windows (scroll_region_ok == 0).  
+   scroll windows (scroll_region_ok == 0).
 
    WINDOW_SIZE is the number of lines being considered for scrolling
    and UNCHANGED_AT_TOP is the vpos of the first line being
    considered.  These two arguments can specify any contiguous range
    of lines.  */
- 
+
 static void
-do_scrolling (current_matrix, matrix, window_size, unchanged_at_top)
+do_scrolling (frame, current_matrix, matrix, window_size, unchanged_at_top)
+     struct frame *frame;
      struct glyph_matrix *current_matrix;
      struct matrix_elt *matrix;
      int window_size;
@@ -261,7 +262,7 @@ do_scrolling (current_matrix, matrix, window_size, unchanged_at_top)
   struct queue *queue_start
     = (struct queue *) alloca (current_matrix->nrows * sizeof (struct queue));
   struct queue *queue = queue_start;
-  
+
   char *retained_p = (char *) alloca (window_size * sizeof (char));
   int *copy_from = (int *) alloca (window_size * sizeof (int));
 
@@ -286,7 +287,7 @@ do_scrolling (current_matrix, matrix, window_size, unchanged_at_top)
   while (i > 0 || j > 0)
     {
       p = matrix + i * (window_size + 1) + j;
-      
+
       if (p->insertcost < p->writecost && p->insertcost < p->deletecost)
 	{
 	  /* Insert should be done at vpos i-1, plus maybe some before.
@@ -310,12 +311,12 @@ do_scrolling (current_matrix, matrix, window_size, unchanged_at_top)
 	  /* Set the terminal window, if not done already.  */
  	  if (! terminal_window_p)
 	    {
-	      set_terminal_window (window_size + unchanged_at_top);
+	      set_terminal_window (frame, window_size + unchanged_at_top);
 	      terminal_window_p = 1;
 	    }
 
 	  /* Delete lines on the terminal.  */
-	  ins_del_lines (j + unchanged_at_top, - p->deletecount);
+	  ins_del_lines (frame, j + unchanged_at_top, - p->deletecount);
 	}
       else
 	{
@@ -340,7 +341,7 @@ do_scrolling (current_matrix, matrix, window_size, unchanged_at_top)
       /* Set the terminal window if not yet done.  */
       if (!terminal_window_p)
 	{
-	  set_terminal_window (window_size + unchanged_at_top);
+	  set_terminal_window (frame, window_size + unchanged_at_top);
 	  terminal_window_p = 1;
 	}
 
@@ -349,7 +350,7 @@ do_scrolling (current_matrix, matrix, window_size, unchanged_at_top)
 	  --queue;
 
 	  /* Do the deletion on the terminal.  */
-	  ins_del_lines (queue->pos, queue->count);
+	  ins_del_lines (frame, queue->pos, queue->count);
 
 	  /* All lines in the range deleted become empty in the glyph
 	     matrix.  Assign to them glyph rows that are not retained.
@@ -368,7 +369,7 @@ do_scrolling (current_matrix, matrix, window_size, unchanged_at_top)
 	    }
 	}
       while (queue > queue_start);
-	  
+
     }
 
   for (k = 0; k < window_size; ++k)
@@ -380,9 +381,9 @@ do_scrolling (current_matrix, matrix, window_size, unchanged_at_top)
 
   /* Some sanity checks if GLYPH_DEBUG != 0.  */
   CHECK_MATRIX (current_matrix);
-  
+
   if (terminal_window_p)
-    set_terminal_window (0);
+    set_terminal_window (frame, 0);
 }
 
 
@@ -433,7 +434,7 @@ calculate_direct_scrolling (frame, matrix, window_size, lines_below,
      FRAME_PTR frame;
      /* matrix is of size window_size + 1 on each side.  */
      struct matrix_elt *matrix;
-     int window_size;
+     int window_size, lines_below;
      int *draw_cost;
      int *old_draw_cost;
      int *old_hash;
@@ -441,27 +442,27 @@ calculate_direct_scrolling (frame, matrix, window_size, lines_below,
      int free_at_end;
 {
   register int i, j;
-  int frame_height = FRAME_HEIGHT (frame);
+  int frame_lines = FRAME_LINES (frame);
   register struct matrix_elt *p, *p1;
   register int cost, cost1, delta;
 
   /* first_insert_cost[-I] is the cost of doing the first insert-line
      at a position I lines above the bottom line in the scroll window. */
   int *first_insert_cost
-    = &FRAME_INSERT_COST (frame)[frame_height - 1];
+    = &FRAME_INSERT_COST (frame)[frame_lines - 1];
   int *first_delete_cost
-    = &FRAME_DELETE_COST (frame)[frame_height - 1];
+    = &FRAME_DELETE_COST (frame)[frame_lines - 1];
   int *next_insert_cost
-    = &FRAME_INSERTN_COST (frame)[frame_height - 1];
+    = &FRAME_INSERTN_COST (frame)[frame_lines - 1];
   int *next_delete_cost
-    = &FRAME_DELETEN_COST (frame)[frame_height - 1];
+    = &FRAME_DELETEN_COST (frame)[frame_lines - 1];
 
   int scroll_overhead;
 
   /* Discourage long scrolls on fast lines.
      Don't scroll nearly a full frame height unless it saves
      at least 1/4 second.  */
-  int extra_cost = baud_rate / (10 * 4 * FRAME_HEIGHT (frame));
+  int extra_cost = baud_rate / (10 * 4 * FRAME_LINES (frame));
 
   if (baud_rate <= 0)
     extra_cost = 1;
@@ -469,7 +470,8 @@ calculate_direct_scrolling (frame, matrix, window_size, lines_below,
   /* Overhead of setting the scroll window, plus the extra cost
      cost of scrolling by a distance of one.  The extra cost is
      added once for consistency with the cost vectors */
-  scroll_overhead = scroll_region_cost + extra_cost;
+  scroll_overhead
+    = FRAME_SCROLL_REGION_COST (frame) + extra_cost;
 
   /* initialize the top left corner of the matrix */
   matrix->writecost = 0;
@@ -565,7 +567,7 @@ calculate_direct_scrolling (frame, matrix, window_size, lines_below,
 		   To reduce the cumulative effects of rounding error, we
 		   vary the position at which the difference is computed. */
 		cost1 += first_insert_cost[-j] - first_insert_cost[1-j] +
-		  (delta-1) * (next_insert_cost[-j] - next_insert_cost[1-j]); 
+		  (delta-1) * (next_insert_cost[-j] - next_insert_cost[1-j]);
 	      }
 	    else
 	      {
@@ -573,7 +575,7 @@ calculate_direct_scrolling (frame, matrix, window_size, lines_below,
 		cost += scroll_overhead + first_delete_cost[-delta] +
 		  (delta-1) * (next_delete_cost[-delta] + extra_cost);
 		cost1 += first_delete_cost[-i] - first_delete_cost[1-i] +
-		  (delta-1) * ( next_delete_cost[-i] - next_delete_cost[1-i]); 
+		  (delta-1) * ( next_delete_cost[-i] - next_delete_cost[1-i]);
 	      }
 	    if (cost1 < cost)
 	      {
@@ -644,7 +646,7 @@ calculate_direct_scrolling (frame, matrix, window_size, lines_below,
    and UNCHANGED_AT_TOP is the vpos of the first line being
    considered.  These two arguments can specify any contiguous range
    of lines.
- 
+
    In the direct scrolling method, a new scroll window is selected
    before each insertion or deletion, so that groups of lines can be
    scrolled directly to their final vertical positions.  This method
@@ -652,8 +654,9 @@ calculate_direct_scrolling (frame, matrix, window_size, lines_below,
    the cost matrix for this approach is constructed. */
 
 static void
-do_direct_scrolling (current_matrix, cost_matrix, window_size,
-		     unchanged_at_top)
+do_direct_scrolling (frame, current_matrix, cost_matrix,
+                     window_size, unchanged_at_top)
+     struct frame *frame;
      struct glyph_matrix *current_matrix;
      struct matrix_elt *cost_matrix;
      int window_size;
@@ -704,11 +707,11 @@ do_direct_scrolling (current_matrix, cost_matrix, window_size,
      without also decrementing i, this corresponds to omitting these
      lines in the new rows, i.e. rows are deleted.  */
   i = j = window_size;
-  
+
   while (i > 0 || j > 0)
     {
       p = cost_matrix + i * (window_size + 1) + j;
-      
+
       if (p->insertcost < p->writecost
 	  && p->insertcost < p->deletecost
 	  && (write_follows_p || i < j))
@@ -720,7 +723,7 @@ do_direct_scrolling (current_matrix, cost_matrix, window_size,
 	  queue->window = i;
 	  queue->pos = i - p->insertcount;
 	  ++queue;
-	  
+
 	  i -= p->insertcount;
 	  write_follows_p = 0;
 	}
@@ -739,14 +742,14 @@ do_direct_scrolling (current_matrix, cost_matrix, window_size,
 	     place they belong.  */
 	  int n_to_write = p->writecount;
 	  write_follows_p = 1;
-	  xassert (n_to_write > 0); 
+	  xassert (n_to_write > 0);
 
 	  if (i > j)
 	    {
 	      /* Immediately insert lines */
-	      set_terminal_window (i + unchanged_at_top);
+	      set_terminal_window (frame, i + unchanged_at_top);
 	      terminal_window_p = 1;
-	      ins_del_lines (j - n_to_write + unchanged_at_top, i - j);
+	      ins_del_lines (frame, j - n_to_write + unchanged_at_top, i - j);
 	    }
 	  else if (i < j)
 	    {
@@ -776,9 +779,9 @@ do_direct_scrolling (current_matrix, cost_matrix, window_size,
 	  --queue;
 	  if (queue->count)
 	    {
-	      set_terminal_window (queue->window);
+	      set_terminal_window (frame, queue->window);
 	      terminal_window_p = 1;
-	      ins_del_lines (queue->pos, queue->count);
+	      ins_del_lines (frame, queue->pos, queue->count);
 	    }
 	  else
 	    {
@@ -801,7 +804,7 @@ do_direct_scrolling (current_matrix, cost_matrix, window_size,
 		       copy_from, retained_p);
 
   if (terminal_window_p)
-    set_terminal_window (0);
+    set_terminal_window (frame, 0);
 }
 
 
@@ -821,13 +824,13 @@ scrolling_1 (frame, window_size, unchanged_at_top, unchanged_at_bottom,
   matrix = ((struct matrix_elt *)
 	    alloca ((window_size + 1) * (window_size + 1) * sizeof *matrix));
 
-  if (scroll_region_ok)
+  if (FRAME_SCROLL_REGION_OK (frame))
     {
       calculate_direct_scrolling (frame, matrix, window_size,
 				  unchanged_at_bottom,
-				  draw_cost, old_draw_cost, 
+				  draw_cost, old_draw_cost,
 				  old_hash, new_hash, free_at_end);
-      do_direct_scrolling (frame->current_matrix,
+      do_direct_scrolling (frame, frame->current_matrix,
 			   matrix, window_size, unchanged_at_top);
     }
   else
@@ -835,7 +838,8 @@ scrolling_1 (frame, window_size, unchanged_at_top, unchanged_at_bottom,
       calculate_scrolling (frame, matrix, window_size, unchanged_at_bottom,
 			   draw_cost, old_hash, new_hash,
 			   free_at_end);
-      do_scrolling (frame->current_matrix, matrix, window_size,
+      do_scrolling (frame,
+                    frame->current_matrix, matrix, window_size,
 		    unchanged_at_top);
     }
 }
@@ -912,12 +916,12 @@ scroll_cost (frame, from, to, amount)
      will not be involved in actual motion.  */
   int limit = to;
   int offset;
-  int height = FRAME_HEIGHT (frame);
+  int height = FRAME_LINES (frame);
 
   if (amount == 0)
     return 0;
 
-  if (! scroll_region_ok)
+  if (! FRAME_SCROLL_REGION_OK (frame))
     limit = height;
   else if (amount > 0)
     limit += amount;
@@ -950,11 +954,11 @@ line_ins_del (frame, ov1, pf1, ovn, pfn, ov, mf)
      register int *ov, *mf;
 {
   register int i;
-  register int frame_height = FRAME_HEIGHT (frame);
+  register int frame_lines = FRAME_LINES (frame);
   register int insert_overhead = ov1 * 10;
   register int next_insert_cost = ovn * 10;
 
-  for (i = frame_height-1; i >= 0; i--)
+  for (i = frame_lines-1; i >= 0; i--)
     {
       mf[i] = next_insert_cost / 10;
       next_insert_cost += pfn;
@@ -1003,12 +1007,12 @@ ins_del_costs (frame,
    only) and those that must repeatedly insert one line.
 
    The cost to insert N lines at line L is
-   	    [tt.t_ILov  + (frame_height + 1 - L) * tt.t_ILpf] +
-	N * [tt.t_ILnov + (frame_height + 1 - L) * tt.t_ILnpf]
+   	    [tt.t_ILov  + (frame_lines + 1 - L) * tt.t_ILpf] +
+	N * [tt.t_ILnov + (frame_lines + 1 - L) * tt.t_ILnpf]
 
    ILov represents the basic insert line overhead.  ILpf is the padding
    required to allow the terminal time to move a line: insertion at line
-   L changes (frame_height + 1 - L) lines.
+   L changes (frame_lines + 1 - L) lines.
 
    The first bracketed expression above is the overhead; the second is
    the multiply factor.  Both are dependent only on the position at
@@ -1038,27 +1042,27 @@ do_line_insertion_deletion_costs (frame,
     {
       FRAME_INSERT_COST (frame) =
 	(int *) xrealloc (FRAME_INSERT_COST (frame),
-			  FRAME_HEIGHT (frame) * sizeof (int));
+			  FRAME_LINES (frame) * sizeof (int));
       FRAME_DELETEN_COST (frame) =
 	(int *) xrealloc (FRAME_DELETEN_COST (frame),
-			  FRAME_HEIGHT (frame) * sizeof (int));
+			  FRAME_LINES (frame) * sizeof (int));
       FRAME_INSERTN_COST (frame) =
 	(int *) xrealloc (FRAME_INSERTN_COST (frame),
-			  FRAME_HEIGHT (frame) * sizeof (int));
+			  FRAME_LINES (frame) * sizeof (int));
       FRAME_DELETE_COST (frame) =
 	(int *) xrealloc (FRAME_DELETE_COST (frame),
-			  FRAME_HEIGHT (frame) * sizeof (int));
+			  FRAME_LINES (frame) * sizeof (int));
     }
   else
     {
       FRAME_INSERT_COST (frame) =
-	(int *) xmalloc (FRAME_HEIGHT (frame) * sizeof (int));
+	(int *) xmalloc (FRAME_LINES (frame) * sizeof (int));
       FRAME_DELETEN_COST (frame) =
-	(int *) xmalloc (FRAME_HEIGHT (frame) * sizeof (int));
+	(int *) xmalloc (FRAME_LINES (frame) * sizeof (int));
       FRAME_INSERTN_COST (frame) =
-	(int *) xmalloc (FRAME_HEIGHT (frame) * sizeof (int));
-      FRAME_DELETE_COST (frame) = 
-	(int *) xmalloc (FRAME_HEIGHT (frame) * sizeof (int));
+	(int *) xmalloc (FRAME_LINES (frame) * sizeof (int));
+      FRAME_DELETE_COST (frame) =
+	(int *) xmalloc (FRAME_LINES (frame) * sizeof (int));
     }
 
   ins_del_costs (frame,
@@ -1072,3 +1076,6 @@ do_line_insertion_deletion_costs (frame,
 		 FRAME_DELETE_COST (frame), FRAME_DELETEN_COST (frame),
 		 coefficient);
 }
+
+/* arch-tag: cdb7149c-48e7-4793-a948-2786c8e45485
+   (do not change this comment) */
